#include <stdlib.h>
#include <stdio.h>
#include <ncurses.h>
#include <string>
#include <vector>

#include "Util.h"
#include "Frame.h"

using namespace std;

extern FILE * ferr;

Frame::Frame()
{
    printf("Frame()\n");
}

Frame::~Frame()
{

    if (fin)
    {
        // Close the file.
        fclose(fin);
    }
    if ( win )
        delwin(win);
}

Frame::Frame(char* name, unsigned int n, unsigned int total)
{
    number=n;
    totalFrames=total;
    strncpy( fileName, name, FILE_NAME_SIZE);

    // Derive other info
    lines=LINES/total; // Total lines for this frame.
    prevOpened=0;
    actualBufferSize=0;
    bufferSize=100;
    startLine=(LINES/totalFrames) * number; // Where to move to

    fin=NULL;
    win=NULL;

    fprintf(ferr,"lines=%d;number=%d;prevOpened=%d;bufferSize=%d;totalFrames=%d\
        startLine=%d\n",lines, number,prevOpened,bufferSize,totalFrames,startLine);

}

int Frame::setVariables()
{
    startLine=(LINES/totalFrames) * number ;
    lines=LINES/totalFrames;
}

Frame::reSize(unsigned int l )
{
    printf("reSize(%d)\n",l);
}

// Take a parameter to determine how far back to read.
unsigned long Frame::findPos(unsigned long maxLines=0)
{
    long i=-1;
    unsigned long cr_count=0;
    char c;
    maxLines = maxLines?maxLines:lines;

    // Assume file is open.
    if (!fin) clean_exit(1,"Frame::findPos File not open yet!");


    while (    cr_count < maxLines  // defaults to lines.
            && (fseek(fin, i, SEEK_END)==0) 
            && !feof(fin) )
    {

        c=getc(fin);
        if (c == '\n')
            cr_count++;

        i--;
    }
    // Go back to the last char read...
    fseek(fin, i+1, SEEK_END);

    return ftell(fin);
}

Frame::print()
{
    int j,i;
    string Buf;
    char c;
    // Only do this if we haven't initialized our buffer.
    if ( !prevOpened )
    {
        fprintf(ferr,"%d Not prevOpened\n",number);

        fprintf(ferr,"Opening %s...\n",fileName);
        fflush(ferr);
        fin = fopen(fileName, "r" );

        fprintf(ferr,"Opened %s...\n",fileName);
        fflush(ferr);

        if ( !fin ) 
        {
            fprintf(stderr,"Error opening <%s>\n", fileName);
            fflush(ferr);
            clean_exit(1, "Frame::print() Error opening file!");
        }

        fprintf(ferr,"apparenly %s is readable...\n",fileName);
        fflush(ferr);
        // Set our starting point at however far back we are to read.
        findPos(bufferSize);
        fprintf(ferr,"Found our pos...\n");
        fflush(ferr);

        // Open our new windows.
        win=newwin(lines,0,startLine,0 );
        prevOpened=1;
        fprintf(ferr,"Windows opened....\n");
        fflush(ferr);
    }
    else
    {
        fprintf(ferr,"%d prevOpened\n",number);
        fflush(ferr);
    }

    // If the screen size has change, reset our variables.
    setVariables();

    // Read the data in.
    // Our file pos should have already been set, read 
    // til EOF.
    if (feof(fin) )
        fprintf(ferr,"\tfin was empty\n");
    fflush(ferr);

    while ( !feof(fin) )
    {
        j=0;
        Buf = "";
        while( (c = getc(fin)) != '\n' && !feof(fin) )
        {
            j++;
            Buf += c;

        }
        fprintf(ferr,"%d: read <%s>\n",number, Buf.c_str());
        fflush(ferr);
        if ( Buf != "" )
            Screen.push_back(Buf);

    }

    // Clear the oef error..
    clearerr(fin);

    // Print the title, in reverse!
    //move (startLine,0); deleteln();
    werase(win); // clear the screen.
    wmove (win,0,0);
    wattron(win,A_REVERSE);
    wprintw(win,"File: <%s>",fileName);
    wattroff(win,A_REVERSE);

    // Set our line in buffer to start from...
    if ( Screen.size() < lines )
        i=0;
    else
        i=Screen.size() - lines+1 ;

    //fprintf(ferr, "i=%d\n",i);
    for ( j=1; j<lines && i < Screen.size(); j++)
    {
        // Move to the next line.
        wmove (win,j,0); 
        wprintw(win,"%s",Screen[i].c_str());
        //wprintw(win,"Hello world.\n");
        i++;
    }
    wrefresh(win);

    // Save where we left off.
    prevPosition = ftell(fin);
}
